@@ -8,6 +8,7 @@
 import warnings
 
 from mutagen._util import DictMixin
+from mutagen._compat import izip
 
 
 class FileType(DictMixin):
@@ -88,7 +89,20 @@ class FileType(DictMixin):
             return self.tags.keys()
 
     def delete(self, filename=None):
-        """Remove tags from a file."""
+        """Remove tags from a file.
+
+        In cases where the tagging format is independent of the file type
+        (for example `mutagen.ID3`) all traces of the tagging format will
+        be removed.
+        In cases where the tag is part of the file type, all tags and
+        padding will be removed.
+
+        The tags attribute will be cleared as well if there is one.
+
+        Does nothing if the file has no tags.
+
+        :raises mutagen.MutagenError: if deleting wasn't possible
+        """
 
         if self.tags is not None:
             if filename is None:
@@ -100,7 +114,10 @@ class FileType(DictMixin):
             return self.tags.delete(filename)
 
     def save(self, filename=None, **kwargs):
-        """Save metadata tags."""
+        """Save metadata tags.
+
+        :raises mutagen.MutagenError: if saving wasn't possible
+        """
 
         if filename is None:
             filename = self.filename
@@ -126,7 +143,8 @@ class FileType(DictMixin):
     def add_tags(self):
         """Adds new tags to the file.
 
-        Raises if tags already exist.
+        :raises mutagen.MutagenError: if tags already exist or adding is not
+            possible.
         """
 
         raise NotImplementedError
@@ -218,17 +236,15 @@ def File(filename, options=None, easy=False):
     if not options:
         return None
 
-    fileobj = open(filename, "rb")
-    try:
+    with open(filename, "rb") as fileobj:
         header = fileobj.read(128)
         # Sort by name after score. Otherwise import order affects
         # Kind sort order, which affects treatment of things with
         # equals scores.
         results = [(Kind.score(filename, fileobj, header), Kind.__name__)
                    for Kind in options]
-    finally:
-        fileobj.close()
-    results = list(zip(results, options))
+
+    results = list(izip(results, options))
     results.sort()
     (score, name), Kind = results[-1]
     if score > 0:
