@@ -1,6 +1,7 @@
-# Vorbis comment support for Mutagen
-# Copyright 2005-2006 Joe Wreschnig
-#           2013 Christoph Reiter
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2005-2006  Joe Wreschnig
+#                    2013  Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
@@ -74,6 +75,7 @@ class VComment(mutagen.Metadata, list):
     vendor = u"Mutagen " + mutagen.version_string
 
     def __init__(self, data=None, *args, **kwargs):
+        self._size = 0
         # Collect the args to pass to load, this lets child classes
         # override just load and get equivalent magic for the
         # constructor.
@@ -82,7 +84,9 @@ class VComment(mutagen.Metadata, list):
                 data = BytesIO(data)
             elif not hasattr(data, 'read'):
                 raise TypeError("VComment requires bytes or a file-like")
+            start = data.tell()
             self.load(data, *args, **kwargs)
+            self._size = data.tell() - start
 
     def load(self, fileobj, errors='replace', framing=True):
         """Parse a Vorbis comment from a file-like object.
@@ -127,6 +131,7 @@ class VComment(mutagen.Metadata, list):
                         tag = tag.decode("ascii")
                     if is_valid_key(tag):
                         self.append((tag, value))
+
             if framing and not bytearray(fileobj.read(1))[0] & 0x01:
                 raise VorbisUnsetFrameError("framing bit was unset")
         except (cdata.error, TypeError):
@@ -144,7 +149,7 @@ class VComment(mutagen.Metadata, list):
 
         if not isinstance(self.vendor, text_type):
             if PY3:
-                raise ValueError
+                raise ValueError("vendor needs to be str")
 
             try:
                 self.vendor.decode('utf-8')
@@ -160,7 +165,7 @@ class VComment(mutagen.Metadata, list):
 
             if not isinstance(value, text_type):
                 if PY3:
-                    raise ValueError
+                    raise ValueError("%r needs to be str" % key)
 
                 try:
                     value.decode("utf-8")
