@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Copyright (C) 2005  Michael Urman
 #
 # This program is free software; you can redistribute it and/or modify
@@ -5,19 +7,16 @@
 # published by the Free Software Foundation.
 
 import zlib
-from warnings import warn
 from struct import unpack
 
-from ._util import (
-    ID3Warning, ID3JunkFrameError, ID3BadCompressedData,
-    ID3EncryptionUnsupportedError, ID3BadUnsynchData, unsynch)
+from ._util import ID3JunkFrameError, ID3EncryptionUnsupportedError, unsynch
 from ._specs import (
     BinaryDataSpec, StringSpec, Latin1TextSpec, EncodedTextSpec, ByteSpec,
     EncodingSpec, ASPIIndexSpec, SizedIntegerSpec, IntegerSpec,
     VolumeAdjustmentsSpec, VolumePeakSpec, VolumeAdjustmentSpec,
     ChannelSpec, MultiSpec, SynchronizedTextSpec, KeyEventSpec, TimeStampSpec,
-    EncodedNumericPartTextSpec, EncodedNumericTextSpec)
-from .._compat import text_type, string_types, swap_to_string
+    EncodedNumericPartTextSpec, EncodedNumericTextSpec, SpecError)
+from .._compat import text_type, string_types, swap_to_string, iteritems, izip
 
 
 def is_valid_frame_id(frame_id):
@@ -62,7 +61,7 @@ class Frame(object):
             # ask the sub class to fill in our data
             other._to_other(self)
         else:
-            for checker, val in zip(self._framespec, args):
+            for checker, val in izip(self._framespec, args):
                 setattr(self, checker.name, checker.validate(self, val))
             for checker in self._framespec[len(args):]:
                 try:
@@ -119,20 +118,19 @@ class Frame(object):
         return '%s(%s)' % (type(self).__name__, ', '.join(kw))
 
     def _readData(self, data):
-        odata = data
+        """Raises ID3JunkFrameError; Returns leftover data"""
+
         for reader in self._framespec:
             if len(data):
                 try:
                     value, data = reader.read(self, data)
-                except UnicodeDecodeError:
-                    raise ID3JunkFrameError
+                except SpecError as e:
+                    raise ID3JunkFrameError(e)
             else:
-                raise ID3JunkFrameError
+                raise ID3JunkFrameError("no data left")
             setattr(self, reader.name, value)
-        if data.strip(b'\x00'):
-            warn('Leftover data: %s: %r (from %r)' % (
-                 type(self).__name__, data, odata),
-                 ID3Warning)
+
+        return data
 
     def _writeData(self):
         data = []
@@ -148,10 +146,17 @@ class Frame(object):
         return "[unrepresentable data]"
 
     @classmethod
-    def fromData(cls, id3, tflags, data):
-        """Construct this ID3 frame from raw string data."""
+    def _fromData(cls, id3, tflags, data):
+        """Construct this ID3 frame from raw string data.
 
-        if id3._V24 <= id3.version:
+        Raises:
+
+        ID3JunkFrameError in case parsing failed
+        NotImplementedError in case parsing isn't implemented
+        ID3EncryptionUnsupportedError in case the frame is encrypted.
+        """
+
+        if id3.version >= id3._V24:
             if tflags & (Frame.FLAG24_COMPRESS | Frame.FLAG24_DATALEN):
                 # The data length int is syncsafe in 2.4 (but not 2.3).
                 # However, we don't actually need the data length int,
@@ -162,9 +167,12 @@ class Frame(object):
             if tflags & Frame.FLAG24_UNSYNCH or id3.f_unsynch:
                 try:
                     data = unsynch.decode(data)
-                except ValueError as err:
-                    if id3.PEDANTIC:
-                        raise ID3BadUnsynchData('%s: %r' % (err, data))
+                except ValueError:
+                    # Some things write synch-unsafe data with either the frame
+                    # or global unsynch flag set. Try to load them as is.
+                    # https://bitbucket.org/lazka/mutagen/issue/210
+                    # https://bitbucket.org/lazka/mutagen/issue/223
+                    pass
             if tflags & Frame.FLAG24_ENCRYPT:
                 raise ID3EncryptionUnsupportedError
             if tflags & Frame.FLAG24_COMPRESS:
@@ -177,10 +185,10 @@ class Frame(object):
                     try:
                         data = zlib.decompress(data)
                     except zlib.error as err:
-                        if id3.PEDANTIC:
-                            raise ID3BadCompressedData('%s: %r' % (err, data))
+                        raise ID3JunkFrameError(
+                            'zlib: %s: %r' % (err, data))
 
-        elif id3._V23 <= id3.version:
+        elif id3.version >= id3._V23:
             if tflags & Frame.FLAG23_COMPRESS:
                 usize, = unpack('>L', data[:4])
                 data = data[4:]
@@ -190,12 +198,9 @@ class Frame(object):
                 try:
                     data = zlib.decompress(data)
                 except zlib.error as err:
-                    if id3.PEDANTIC:
-                        raise ID3BadCompressedData('%s: %r' % (err, data))
+                    raise ID3JunkFrameError('zlib: %s: %r' % (err, data))
 
         frame = cls()
-        frame._rawdata = data
-        frame._flags = tflags
         frame._readData(data)
         return frame
 
@@ -233,24 +238,30 @@ class FrameOpt(Frame):
                 setattr(other, checker.name, getattr(self, checker.name))
 
     def _readData(self, data):
-        odata = data
+        """Raises ID3JunkFrameError; Returns leftover data"""
+
         for reader in self._framespec:
             if len(data):
-                value, data = reader.read(self, data)
+                try:
+                    value, data = reader.read(self, data)
+                except SpecError as e:
+                    raise ID3JunkFrameError(e)
             else:
-                raise ID3JunkFrameError
+                raise ID3JunkFrameError("no data left")
             setattr(self, reader.name, value)
+
         if data:
             for reader in self._optionalspec:
                 if len(data):
-                    value, data = reader.read(self, data)
+                    try:
+                        value, data = reader.read(self, data)
+                    except SpecError as e:
+                        raise ID3JunkFrameError(e)
                 else:
                     break
                 setattr(self, reader.name, value)
-        if data.strip(b'\x00'):
-            warn('Leftover data: %s: %r (from %r)' % (
-                 type(self).__name__, data, odata),
-                 ID3Warning)
+
+        return data
 
     def _writeData(self):
         data = []
@@ -1592,18 +1603,6 @@ class ASPI(Frame):
     __hash__ = Frame.__hash__
 
 
-Frames = dict([(k, v) for (k, v) in globals().items()
-               if len(k) == 4 and isinstance(v, type) and
-               issubclass(v, Frame)])
-"""All supported ID3v2 frames, keyed by frame name."""
-
-try:
-    del(k)
-    del(v)
-except NameError:
-    pass
-
-
 # ID3v2.2 frames
 class UFI(UFID):
     "Unique File Identifier"
@@ -1901,9 +1900,23 @@ class LNK(LINK):
             other.data = self.data
 
 
-Frames_2_2 = dict([(k, v) for (k, v) in globals().items()
-                   if len(k) == 3 and isinstance(v, type) and
-                   issubclass(v, Frame)])
+Frames = {}
+"""All supported ID3v2.3/4 frames, keyed by frame name."""
+
+
+Frames_2_2 = {}
+"""All supported ID3v2.2 frames, keyed by frame name."""
+
+
+k, v = None, None
+for k, v in iteritems(globals()):
+    if isinstance(v, type) and issubclass(v, Frame):
+        v.__module__ = "mutagen.id3"
+
+        if len(k) == 3:
+            Frames_2_2[k] = v
+        elif len(k) == 4:
+            Frames[k] = v
 
 try:
     del k
