@@ -1,5 +1,6 @@
-# MP3 stream header information support for Mutagen.
-# Copyright 2006 Joe Wreschnig
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2006  Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
@@ -10,9 +11,10 @@
 import os
 import struct
 
-from ._compat import endswith
+from ._compat import endswith, xrange
+from ._mp3util import XingHeader, XingHeaderError, VBRIHeader, VBRIHeaderError
 from mutagen import StreamInfo
-from mutagen._util import MutagenError
+from mutagen._util import MutagenError, enum
 from mutagen.id3 import ID3FileType, BitPaddedInt, delete
 
 __all__ = ["MP3", "Open", "delete", "MP3"]
@@ -30,8 +32,47 @@ class InvalidMPEGHeader(error, IOError):
     pass
 
 
+@enum
+class BitrateMode(object):
+
+    UNKNOWN = 0
+    """Probably a CBR file, but not sure"""
+
+    CBR = 1
+    """Constant Bitrate"""
+
+    VBR = 2
+    """Variable Bitrate"""
+
+    ABR = 3
+    """Average Bitrate (a variant of VBR)"""
+
+
+def _guess_xing_bitrate_mode(xing):
+
+    if xing.lame_header:
+        lame = xing.lame_header
+        if lame.vbr_method in (1, 8):
+            return BitrateMode.CBR
+        elif lame.vbr_method in (2, 9):
+            return BitrateMode.ABR
+        elif lame.vbr_method in (3, 4, 5, 6):
+            return BitrateMode.VBR
+        # everything else undefined, continue guessing
+
+    # info tags get only written by lame for cbr files
+    if xing.is_info:
+        return BitrateMode.CBR
+
+    # older lame and non-lame with some variant of vbr
+    if xing.vbr_scale != -1 or xing.lame_version:
+        return BitrateMode.VBR
+
+    return BitrateMode.UNKNOWN
+
+
 # Mode values.
-STEREO, JOINTSTEREO, DUALCHANNEL, MONO = range(4)
+STEREO, JOINTSTEREO, DUALCHANNEL, MONO = xrange(4)
 
 
 class MPEGInfo(StreamInfo):
@@ -46,8 +87,18 @@ class MPEGInfo(StreamInfo):
     Useful attributes:
 
     * length -- audio length, in seconds
+    * channels -- number of audio channels
     * bitrate -- audio bitrate, in bits per second
     * sketchy -- if true, the file may not be valid MPEG audio
+    * encoder_info -- a string containing encoder name and possibly version.
+                      In case a lame tag is present this will start with
+                      ``"LAME "``, if unknown it is empty, otherwise the
+                      text format is undefined.
+    * bitrate_mode -- a :class:`BitrateMode`
+
+    * track_gain -- replaygain track gain (89db) or None
+    * track_peak -- replaygain track peak or None
+    * album_gain -- replaygain album gain (89db) or None
 
     Useless attributes:
 
@@ -74,7 +125,7 @@ class MPEGInfo(StreamInfo):
     }
 
     __BITRATE[(2, 3)] = __BITRATE[(2, 2)]
-    for i in range(1, 4):
+    for i in xrange(1, 4):
         __BITRATE[(2.5, i)] = __BITRATE[(2, i)]
 
     # Map version to sample rates.
@@ -85,6 +136,9 @@ class MPEGInfo(StreamInfo):
     }
 
     sketchy = False
+    encoder_info = u""
+    bitrate_mode = BitrateMode.UNKNOWN
+    track_gain = track_peak = album_gain = album_peak = None
 
     def __init__(self, fileobj, offset=None):
         """Parse MPEG stream information from a file-like object.
@@ -108,7 +162,7 @@ class MPEGInfo(StreamInfo):
             try:
                 id3, insize = struct.unpack('>3sxxx4s', idata)
             except struct.error:
-                id3, insize = '', 0
+                id3, insize = b'', 0
             insize = BitPaddedInt(insize)
             if id3 == b'ID3' and insize > 0:
                 offset = insize + 10
@@ -143,9 +197,9 @@ class MPEGInfo(StreamInfo):
         data = fileobj.read(32768)
 
         frame_1 = data.find(b"\xff")
-        while 0 <= frame_1 <= len(data) - 4:
+        while 0 <= frame_1 <= (len(data) - 4):
             frame_data = struct.unpack(">I", data[frame_1:frame_1 + 4])[0]
-            if (frame_data >> 16) & 0xE0 != 0xE0:
+            if ((frame_data >> 16) & 0xE0) != 0xE0:
                 frame_1 = data.find(b"\xff", frame_1 + 2)
             else:
                 version = (frame_data >> 19) & 0x3
@@ -168,6 +222,8 @@ class MPEGInfo(StreamInfo):
         else:
             raise HeaderNotFoundError("can't sync to an MPEG frame")
 
+        self.channels = 1 if self.mode == MONO else 2
+
         # There is a serious problem here, which is that many flags
         # in an MPEG header are backwards.
         self.version = [2.5, None, 2, 1][version]
@@ -181,13 +237,13 @@ class MPEGInfo(StreamInfo):
 
         if self.layer == 1:
             frame_length = (
-                12 * self.bitrate // self.sample_rate + padding) * 4
+                (12 * self.bitrate // self.sample_rate) + padding) * 4
             frame_size = 384
         elif self.version >= 2 and self.layer == 3:
-            frame_length = 72 * self.bitrate // self.sample_rate + padding
+            frame_length = (72 * self.bitrate // self.sample_rate) + padding
             frame_size = 576
         else:
-            frame_length = 144 * self.bitrate // self.sample_rate + padding
+            frame_length = (144 * self.bitrate // self.sample_rate) + padding
             frame_size = 1152
 
         if check_second:
@@ -199,52 +255,70 @@ class MPEGInfo(StreamInfo):
                     ">H", data[possible:possible + 2])[0]
             except struct.error:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
-            if frame_data & 0xFFE0 != 0xFFE0:
+            if (frame_data & 0xFFE0) != 0xFFE0:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
 
         self.length = 8 * real_size / float(self.bitrate)
 
         # Try to find/parse the Xing header, which trumps the above length
         # and bitrate calculation.
-        fileobj.seek(offset, 0)
-        data = fileobj.read(32768)
+
+        if self.layer != 3:
+            return
+
+        # Xing
+        xing_offset = XingHeader.get_offset(self)
+        fileobj.seek(offset + frame_1 + xing_offset, 0)
         try:
-            xing = data[:-4].index(b"Xing")
-        except ValueError:
-            # Try to find/parse the VBRI header, which trumps the above length
-            # calculation.
-            try:
-                vbri = data[:-24].index(b"VBRI")
-            except ValueError:
-                pass
-            else:
-                # If a VBRI header was found, this is definitely MPEG audio.
-                self.sketchy = False
-                vbri_version = struct.unpack('>H', data[vbri + 4:vbri + 6])[0]
-                if vbri_version == 1:
-                    frame_count = struct.unpack(
-                        '>I', data[vbri + 14:vbri + 18])[0]
-                    samples = float(frame_size * frame_count)
-                    self.length = (samples / self.sample_rate) or self.length
+            xing = XingHeader(fileobj)
+        except XingHeaderError:
+            pass
         else:
-            # If a Xing header was found, this is definitely MPEG audio.
+            lame = xing.lame_header
             self.sketchy = False
-            flags = struct.unpack('>I', data[xing + 4:xing + 8])[0]
-            if flags & 0x1:
-                frame_count = struct.unpack('>I', data[xing + 8:xing + 12])[0]
-                samples = float(frame_size * frame_count)
-                self.length = (samples / self.sample_rate) or self.length
-            if flags & 0x2:
-                bitrate_data = struct.unpack(
-                    '>I', data[xing + 12:xing + 16])[0]
-                self.bitrate = int((bitrate_data * 8) // self.length)
+            self.bitrate_mode = _guess_xing_bitrate_mode(xing)
+            if xing.frames != -1:
+                samples = frame_size * xing.frames
+                if lame is not None:
+                    samples -= lame.encoder_delay_start
+                    samples -= lame.encoder_padding_end
+                self.length = float(samples) / self.sample_rate
+            if xing.bytes != -1 and self.length:
+                self.bitrate = int((xing.bytes * 8) / self.length)
+            if xing.lame_version:
+                self.encoder_info = u"LAME %s" % xing.lame_version
+            if lame is not None:
+                self.track_gain = lame.track_gain_adjustment
+                self.track_peak = lame.track_peak
+                self.album_gain = lame.album_gain_adjustment
+            return
+
+        # VBRI
+        vbri_offset = VBRIHeader.get_offset(self)
+        fileobj.seek(offset + frame_1 + vbri_offset, 0)
+        try:
+            vbri = VBRIHeader(fileobj)
+        except VBRIHeaderError:
+            pass
+        else:
+            self.bitrate_mode = BitrateMode.VBR
+            self.encoder_info = u"FhG"
+            self.sketchy = False
+            self.length = float(frame_size * vbri.frames) / self.sample_rate
+            if self.length:
+                self.bitrate = int((vbri.bytes * 8) / self.length)
 
     def pprint(self):
-        s = "MPEG %s layer %d, %d bps, %s Hz, %.2f seconds" % (
-            self.version, self.layer, self.bitrate, self.sample_rate,
-            self.length)
+        info = str(self.bitrate_mode).split(".", 1)[-1]
+        if self.bitrate_mode == BitrateMode.UNKNOWN:
+            info = u"CBR?"
+        if self.encoder_info:
+            info += ", %s" % self.encoder_info
+        s = u"MPEG %s layer %d, %d bps (%s), %s Hz, %d chn, %.2f seconds" % (
+            self.version, self.layer, self.bitrate, info,
+            self.sample_rate, self.channels, self.length)
         if self.sketchy:
-            s += " (sketchy)"
+            s += u" (sketchy)"
         return s
 
 
