@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2006 Lukas Lalinsky <lalinsky@gmail.com>
-# Copyright 2012 Christoph Reiter <christoph.reiter@gmx.at>
+# Copyright (C) 2006  Lukas Lalinsky
+# Copyright (C) 2012  Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -120,8 +120,8 @@ class MusepackInfo(StreamInfo):
         mandatory_packets = [b"SH", b"RG"]
 
         def check_frame_key(key):
-            if (len(frame_type) != key_size) or \
-                    (not b'AA' <= frame_type <= b'ZZ'):
+            if ((len(frame_type) != key_size) or
+                    (not b'AA' <= frame_type <= b'ZZ')):
                 raise MusepackHeaderError("Invalid frame key.")
 
         frame_type = fileobj.read(key_size)
@@ -133,6 +133,7 @@ class MusepackInfo(StreamInfo):
             except (EOFError, ValueError):
                 raise MusepackHeaderError("Invalid packet size.")
             data_size = frame_size - key_size - slen
+            # packets can be at maximum data_size big and are padded with zeros
 
             if frame_type == b"SH":
                 mandatory_packets.remove(frame_type)
@@ -154,30 +155,36 @@ class MusepackInfo(StreamInfo):
         self.bitrate = 0
 
     def __parse_stream_header(self, fileobj, data_size):
+        # skip CRC
         fileobj.seek(4, 1)
+        remaining_size = data_size - 4
+
         try:
             self.version = bytearray(fileobj.read(1))[0]
         except TypeError:
             raise MusepackHeaderError("SH packet ended unexpectedly.")
+
+        remaining_size -= 1
+
         try:
             samples, l1 = _parse_sv8_int(fileobj)
             samples_skip, l2 = _parse_sv8_int(fileobj)
         except (EOFError, ValueError):
             raise MusepackHeaderError(
                 "SH packet: Invalid sample counts.")
-        left_size = data_size - 5 - l1 - l2
-        if left_size != 2:
-            raise MusepackHeaderError("Invalid SH packet size.")
-        data = fileobj.read(left_size)
-        if len(data) != left_size:
-            raise MusepackHeaderError("SH packet ended unexpectedly.")
-        self.sample_rate = RATES[bytearray(data)[-2] >> 5]
-        self.channels = (bytearray(data)[-1] >> 4) + 1
+
         self.samples = samples - samples_skip
+        remaining_size -= l1 + l2
+
+        data = fileobj.read(remaining_size)
+        if len(data) != remaining_size:
+            raise MusepackHeaderError("SH packet ended unexpectedly.")
+        self.sample_rate = RATES[bytearray(data)[0] >> 5]
+        self.channels = (bytearray(data)[1] >> 4) + 1
 
     def __parse_replaygain_packet(self, fileobj, data_size):
         data = fileobj.read(data_size)
-        if data_size != 9:
+        if data_size < 9:
             raise MusepackHeaderError("Invalid RG packet size.")
         if len(data) != data_size:
             raise MusepackHeaderError("RG packet ended unexpectedly.")
@@ -239,12 +246,12 @@ class MusepackInfo(StreamInfo):
     def pprint(self):
         rg_data = []
         if hasattr(self, "title_gain"):
-            rg_data.append("%+0.2f (title)" % self.title_gain)
+            rg_data.append(u"%+0.2f (title)" % self.title_gain)
         if hasattr(self, "album_gain"):
-            rg_data.append("%+0.2f (album)" % self.album_gain)
+            rg_data.append(u"%+0.2f (album)" % self.album_gain)
         rg_data = (rg_data and ", Gain: " + ", ".join(rg_data)) or ""
 
-        return "Musepack SV%d, %.2f seconds, %d Hz, %d bps%s" % (
+        return u"Musepack SV%d, %.2f seconds, %d Hz, %d bps%s" % (
             self.version, self.length, self.sample_rate, self.bitrate, rg_data)
 
 
