@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2006 Lukas Lalinsky <lalinsky@gmail.com>
+# Copyright (C) 2006  Lukas Lalinsky
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -57,8 +57,8 @@ class OptimFROGInfo(StreamInfo):
             self.length = 0.0
 
     def pprint(self):
-        return "OptimFROG, %.2f seconds, %d Hz" % (self.length,
-                                                   self.sample_rate)
+        return u"OptimFROG, %.2f seconds, %d Hz" % (self.length,
+                                                    self.sample_rate)
 
 
 class OptimFROG(APEv2File):
