@@ -1,13 +1,27 @@
+# -*- coding: utf-8 -*-
+
 import sys
 
-from tests import TestCase, add
+from tests import TestCase
 
 from mutagen._compat import PY2, PY3, text_type
-from mutagen.id3 import BitPaddedInt
+from mutagen.id3 import BitPaddedInt, BitPaddedLong, unsynch
+from mutagen.id3._specs import SpecError
 
 
 class SpecSanityChecks(TestCase):
 
+    def test_aspiindexspec(self):
+        from mutagen.id3 import ASPIIndexSpec
+        from mutagen.id3 import ASPI
+
+        frame = ASPI(b=16, N=2)
+        s = ASPIIndexSpec('name')
+        self.assertRaises(SpecError, s.read, frame, b'')
+        self.assertEqual(s.read(frame, b'\x01\x00\x00\x01'), ([256, 1], b""))
+        frame = ASPI(b=42)
+        self.assertRaises(SpecError, s.read, frame, b'')
+
     def test_bytespec(self):
         from mutagen.id3 import ByteSpec
         s = ByteSpec('name')
@@ -19,8 +33,8 @@ class SpecSanityChecks(TestCase):
     def test_encodingspec(self):
         from mutagen.id3 import EncodingSpec
         s = EncodingSpec('name')
-        self.assertEquals((0, b'abcdefg'), s.read(None, b'abcdefg'))
         self.assertEquals((3, b'abcdefg'), s.read(None, b'\x03abcdefg'))
+        self.assertRaises(SpecError, s.read, None, b'\x04abcdefg')
         self.assertEquals(b'\x00', s.write(None, 0))
         self.assertRaises(TypeError, s.write, None, b'abc')
         self.assertRaises(TypeError, s.write, None, None)
@@ -28,24 +42,26 @@ class SpecSanityChecks(TestCase):
     def test_stringspec(self):
         from mutagen.id3 import StringSpec
         s = StringSpec('name', 3)
-        self.assertEquals((b'abc', b'defg'),  s.read(None, b'abcdefg'))
-        self.assertEquals(b'abc', s.write(None, b'abcdefg'))
+        self.assertEquals(('abc', b'defg'), s.read(None, b'abcdefg'))
+        self.assertEquals(b'abc', s.write(None, 'abcdefg'))
         self.assertEquals(b'\x00\x00\x00', s.write(None, None))
-        self.assertEquals(b'\x00\x00\x00', s.write(None, b'\x00'))
-        self.assertEquals(b'a\x00\x00', s.write(None, b'a'))
+        self.assertEquals(b'\x00\x00\x00', s.write(None, '\x00'))
+        self.assertEquals(b'a\x00\x00', s.write(None, 'a'))
+        self.assertRaises(SpecError, s.read, None, b'\xff')
 
     def test_binarydataspec(self):
         from mutagen.id3 import BinaryDataSpec
         s = BinaryDataSpec('name')
         self.assertEquals((b'abcdefg', b''), s.read(None, b'abcdefg'))
-        self.assertEquals(b'',  s.write(None, None))
-        self.assertEquals(b'43',  s.write(None, 43))
-        self.assertEquals(b'abc',  s.write(None, b'abc'))
+        self.assertEquals(b'', s.write(None, None))
+        self.assertEquals(b'43', s.write(None, 43))
+        self.assertEquals(b'abc', s.write(None, b'abc'))
 
     def test_encodedtextspec(self):
         from mutagen.id3 import EncodedTextSpec, Frame
         s = EncodedTextSpec('name')
-        f = Frame(); f.encoding = 0
+        f = Frame()
+        f.encoding = 0
         self.assertEquals((u'abcd', b'fg'), s.read(f, b'abcd\x00fg'))
         self.assertEquals(b'abcdefg\x00', s.write(f, u'abcdefg'))
         self.assertRaises(AttributeError, s.write, f, None)
@@ -53,7 +69,8 @@ class SpecSanityChecks(TestCase):
     def test_timestampspec(self):
         from mutagen.id3 import TimeStampSpec, Frame, ID3TimeStamp
         s = TimeStampSpec('name')
-        f = Frame(); f.encoding = 0
+        f = Frame()
+        f.encoding = 0
         self.assertEquals((ID3TimeStamp('ab'), b'fg'), s.read(f, b'ab\x00fg'))
         self.assertEquals((ID3TimeStamp('1234'), b''), s.read(f, b'1234\x00'))
         self.assertEquals(b'1234\x00', s.write(f, ID3TimeStamp('1234')))
@@ -61,7 +78,7 @@ class SpecSanityChecks(TestCase):
         if PY3:
             self.assertRaises(TypeError, ID3TimeStamp, b"blah")
         self.assertEquals(
-            text_type(ID3TimeStamp(u"2000-01-01")),  u"2000-01-01")
+            text_type(ID3TimeStamp(u"2000-01-01")), u"2000-01-01")
         self.assertEquals(
             bytes(ID3TimeStamp(u"2000-01-01")), b"2000-01-01")
 
@@ -100,9 +117,6 @@ class SpecSanityChecks(TestCase):
         self.assertEquals(s.write(f, [(u"A", 100)]), b"A\x00\x00\x00\x00d")
 
 
-add(SpecSanityChecks)
-
-
 class SpecValidateChecks(TestCase):
 
     def test_volumeadjustmentspec(self):
@@ -120,7 +134,29 @@ class SpecValidateChecks(TestCase):
         s = ByteSpec('byte')
         self.assertRaises(ValueError, s.validate, None, 1000)
 
-add(SpecValidateChecks)
+    def test_stringspec(self):
+        from mutagen.id3 import StringSpec
+        s = StringSpec('byte', 3)
+        self.assertEqual(s.validate(None, None), None)
+        self.assertEqual(s.validate(None, "ABC"), "ABC")
+        self.assertEqual(s.validate(None, u"ABC"), u"ABC")
+        self.assertRaises(ValueError, s.validate, None, "abc2")
+        self.assertRaises(ValueError, s.validate, None, "ab")
+
+        if PY3:
+            self.assertRaises(TypeError, s.validate, None, b"ABC")
+            self.assertRaises(ValueError, s.validate, None, u"\xf6\xe4\xfc")
+
+    def test_binarydataspec(self):
+        from mutagen.id3 import BinaryDataSpec
+        s = BinaryDataSpec('name')
+        self.assertEqual(s.validate(None, None), None)
+        self.assertEqual(s.validate(None, b"abc"), b"abc")
+        if PY3:
+            self.assertRaises(TypeError, s.validate, None, "abc")
+        else:
+            self.assertEqual(s.validate(None, u"abc"), b"abc")
+            self.assertRaises(ValueError, s.validate, None, u"\xf6\xe4\xfc")
 
 
 class NoHashSpec(TestCase):
@@ -129,11 +165,18 @@ class NoHashSpec(TestCase):
         from mutagen.id3 import Spec
         self.failUnlessRaises(TypeError, {}.__setitem__, Spec("foo"), None)
 
-add(NoHashSpec)
-
 
 class BitPaddedIntTest(TestCase):
 
+    def test_long(self):
+        if PY2:
+            data = BitPaddedInt.to_str(sys.maxint + 1, width=16)
+            val = BitPaddedInt(data)
+            self.assertEqual(val, sys.maxint + 1)
+            self.assertTrue(isinstance(val, BitPaddedLong))
+        else:
+            self.assertTrue(BitPaddedInt is BitPaddedLong)
+
     def test_zero(self):
         self.assertEquals(BitPaddedInt(b'\x00\x00\x00\x00'), 0)
 
@@ -141,7 +184,8 @@ class BitPaddedIntTest(TestCase):
         self.assertEquals(BitPaddedInt(b'\x00\x00\x00\x01'), 1)
 
     def test_1l(self):
-        self.assertEquals(BitPaddedInt(b'\x01\x00\x00\x00', bigendian=False), 1)
+        self.assertEquals(
+            BitPaddedInt(b'\x01\x00\x00\x00', bigendian=False), 1)
 
     def test_129(self):
         self.assertEquals(BitPaddedInt(b'\x00\x00\x01\x01'), 0x81)
@@ -154,14 +198,14 @@ class BitPaddedIntTest(TestCase):
 
     def test_32b(self):
         self.assertEquals(BitPaddedInt(b'\xFF\xFF\xFF\xFF', bits=8),
-            0xFFFFFFFF)
+                          0xFFFFFFFF)
 
     def test_32bi(self):
         self.assertEquals(BitPaddedInt(0xFFFFFFFF, bits=8), 0xFFFFFFFF)
 
     def test_s32b(self):
         self.assertEquals(BitPaddedInt(b'\xFF\xFF\xFF\xFF', bits=8).as_str(),
-            b'\xFF\xFF\xFF\xFF')
+                          b'\xFF\xFF\xFF\xFF')
 
     def test_s0(self):
         self.assertEquals(BitPaddedInt.to_str(0), b'\x00\x00\x00\x00')
@@ -192,12 +236,12 @@ class BitPaddedIntTest(TestCase):
     def test_str_int_init(self):
         from struct import pack
         self.assertEquals(BitPaddedInt(238).as_str(),
-                BitPaddedInt(pack('>L', 238)).as_str())
+                          BitPaddedInt(pack('>L', 238)).as_str())
 
     def test_varwidth(self):
         self.assertEquals(len(BitPaddedInt.to_str(100)), 4)
         self.assertEquals(len(BitPaddedInt.to_str(100, width=-1)), 4)
-        self.assertEquals(len(BitPaddedInt.to_str(2**32, width=-1)), 5)
+        self.assertEquals(len(BitPaddedInt.to_str(2 ** 32, width=-1)), 5)
 
     def test_minwidth(self):
         self.assertEquals(
@@ -228,25 +272,39 @@ class BitPaddedIntTest(TestCase):
         self.failIf(BitPaddedInt.has_valid_padding(0x9f << 32, bits=6))
         self.failUnless(BitPaddedInt.has_valid_padding(0x3f << 16, bits=6))
 
-add(BitPaddedIntTest)
-
 
 class TestUnsynch(TestCase):
 
-    def test_unsync_encode(self):
-        from mutagen.id3 import unsynch as un
-        for d in (b'\xff\xff\xff\xff', b'\xff\xf0\x0f\x00', b'\xff\x00\x0f\xf0'):
-            self.assertEquals(d, un.decode(un.encode(d)))
-            self.assertNotEqual(d, un.encode(d))
-        self.assertEquals(b'\xff\x44', un.encode(b'\xff\x44'))
-        self.assertEquals(b'\xff\x00\x00', un.encode(b'\xff\x00'))
+    def test_unsync_encode_decode(self):
+        pairs = [
+            (b'', b''),
+            (b'\x00', b'\x00'),
+            (b'\x44', b'\x44'),
+            (b'\x44\xff', b'\x44\xff\x00'),
+            (b'\xe0', b'\xe0'),
+            (b'\xe0\xe0', b'\xe0\xe0'),
+            (b'\xe0\xff', b'\xe0\xff\x00'),
+            (b'\xff', b'\xff\x00'),
+            (b'\xff\x00', b'\xff\x00\x00'),
+            (b'\xff\x00\x00', b'\xff\x00\x00\x00'),
+            (b'\xff\x01', b'\xff\x01'),
+            (b'\xff\x44', b'\xff\x44'),
+            (b'\xff\xe0', b'\xff\x00\xe0'),
+            (b'\xff\xe0\xff', b'\xff\x00\xe0\xff\x00'),
+            (b'\xff\xf0\x0f\x00', b'\xff\x00\xf0\x0f\x00'),
+            (b'\xff\xff', b'\xff\x00\xff\x00'),
+            (b'\xff\xff\x01', b'\xff\x00\xff\x01'),
+            (b'\xff\xff\xff\xff', b'\xff\x00\xff\x00\xff\x00\xff\x00'),
+        ]
 
-    def test_unsync_decode(self):
-        from mutagen.id3 import unsynch as un
-        self.assertRaises(ValueError, un.decode, b'\xff\xff\xff\xff')
-        self.assertRaises(ValueError, un.decode, b'\xff\xf0\x0f\x00')
-        self.assertRaises(ValueError, un.decode, b'\xff\xe0')
-        self.assertRaises(ValueError, un.decode, b'\xff')
-        self.assertEquals(b'\xff\x44', un.decode(b'\xff\x44'))
+        for d, e in pairs:
+            self.assertEqual(unsynch.encode(d), e)
+            self.assertEqual(unsynch.decode(e), d)
+            self.assertEqual(unsynch.decode(unsynch.encode(e)), e)
+            self.assertEqual(unsynch.decode(e + e), d + d)
 
-add(TestUnsynch)
+    def test_unsync_decode_invalid(self):
+        self.assertRaises(ValueError, unsynch.decode, b'\xff\xff\xff\xff')
+        self.assertRaises(ValueError, unsynch.decode, b'\xff\xf0\x0f\x00')
+        self.assertRaises(ValueError, unsynch.decode, b'\xff\xe0')
+        self.assertRaises(ValueError, unsynch.decode, b'\xff')
