@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes
-from mutagen._util import decode_terminated, dict_match, enum
-from mutagen._util import BitReader, BitReaderError
+from mutagen._util import decode_terminated, dict_match, enum, get_size
+from mutagen._util import BitReader, BitReaderError, resize_bytes
 from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2, \
     cBytesIO, xrange
 from tests import TestCase
@@ -274,6 +274,21 @@ class FileHandling(TestCase):
         fobj.seek(0, 0)
         return fobj.read()
 
+    def test_resize_decrease(self):
+        o = self.file(b'abcd')
+        resize_bytes(o, 2, 1, 1)
+        self.assertEqual(self.read(o), b"abd")
+
+    def test_resize_increase(self):
+        o = self.file(b'abcd')
+        resize_bytes(o, 2, 4, 1)
+        self.assertEqual(self.read(o), b"abcd\x00d")
+
+    def test_resize_nothing(self):
+        o = self.file(b'abcd')
+        resize_bytes(o, 2, 2, 1)
+        self.assertEqual(self.read(o), b"abcd")
+
     def test_insert_into_empty(self):
         o = self.file(b'')
         insert_bytes(o, 8, 0)
@@ -362,7 +377,7 @@ class FileHandling(TestCase):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
         # code for transfers of this or similar sizes.
-        data = u''.join(map(text_type, range(12574)))  # 51760 bytes
+        data = u''.join(map(text_type, xrange(12574)))  # 51760 bytes
         data = data.encode("ascii")
         o = self.file(data)
         insert_bytes(o, 6106, 79)
@@ -372,7 +387,7 @@ class FileHandling(TestCase):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
         # code for transfers of this or similar sizes.
-        data = u''.join(map(text_type, range(12574)))  # 51760 bytes
+        data = u''.join(map(text_type, xrange(12574)))  # 51760 bytes
         data = data.encode("ascii")
         o = self.file(data[:6106 + 79] + data[79:])
         delete_bytes(o, 6106, 79)
@@ -393,13 +408,13 @@ class FileHandling(TestCase):
                         min_change_size < max_change_size and
                         min_buffer_size < max_buffer_size,
                         "Given testing parameters make this test useless")
-        for j in range(num_runs):
+        for j in xrange(num_runs):
             data = b"ABCDEFGHIJKLMNOPQRSTUVWXYZ" * 1024
             fobj = self.file(data)
             filesize = len(data)
             # Generate the list of changes to apply
             changes = []
-            for i in range(num_changes):
+            for i in xrange(num_changes):
                 change_size = random.randrange(
                     min_change_size, max_change_size)
                 change_offset = random.randrange(0, filesize)
@@ -426,21 +441,6 @@ class FileHandling(TestCase):
             self.failUnless(fobj.read() == data)
 
 
-class FileHandlingMockedLock(FileHandling):
-
-    def setUp(self):
-        def MockLockF(*args, **kwargs):
-            raise IOError
-        self._orig_lockf = fcntl.lockf
-        fcntl.lockf = MockLockF
-
-    def tearDown(self):
-        fcntl.lockf = self._orig_lockf
-
-if not fcntl:
-    del FileHandlingMockedLock
-
-
 class FileHandlingMockedMMapMove(FileHandling):
 
     def setUp(self):
@@ -512,6 +512,16 @@ class Tenum(TestCase):
         self.assertTrue(isinstance(repr(Foo.FOO), str))
 
 
+class Tget_size(TestCase):
+
+    def test_get_size(self):
+        f = cBytesIO(b"foo")
+        f.seek(1, 0)
+        self.assertEqual(f.tell(), 1)
+        self.assertEqual(get_size(f), 3)
+        self.assertEqual(f.tell(), 1)
+
+
 class Tdecode_terminated(TestCase):
 
     def test_all(self):
