@@ -10,8 +10,9 @@ from tempfile import mkstemp
 from tests import TestCase, DATA_DIR
 
 import mutagen.apev2
-from mutagen._compat import PY3, text_type
-from mutagen.apev2 import APEv2File, APEv2, is_valid_apev2_key, APEBadItemError
+from mutagen._compat import PY3, text_type, izip, xrange
+from mutagen.apev2 import APEv2File, APEv2, is_valid_apev2_key, \
+    APEBadItemError, error as APEv2Error
 
 
 SAMPLE = os.path.join(DATA_DIR, "click.mpc")
@@ -229,7 +230,7 @@ class TAPEv2(TestCase):
 
         self.failUnlessEqual(
             self.audio.items(),
-            list(zip(self.audio.keys(), self.audio.values())))
+            list(izip(self.audio.keys(), self.audio.values())))
 
     def test_key_type(self):
         key = self.audio.keys()[0]
@@ -351,7 +352,7 @@ class TAPETextValue(TestCase):
         self.value[2] = self.sample[2] = 'baz'
 
     def test_getitem(self):
-        for i in range(len(self.value)):
+        for i in xrange(len(self.value)):
             self.failUnlessEqual(self.sample[i], self.value[i])
 
     def test_delitem(self):
@@ -425,7 +426,7 @@ class TAPEv2File(TestCase):
         self.failUnless(self.audio.tags is None)
         self.audio.add_tags()
         self.failUnless(self.audio.tags is not None)
-        self.failUnlessRaises(ValueError, self.audio.add_tags)
+        self.failUnlessRaises(APEv2Error, self.audio.add_tags)
 
     def test_unknown_info(self):
         info = self.audio.info
