@@ -5,9 +5,13 @@ import shutil
 from tempfile import mkstemp
 from tests import TestCase, DATA_DIR
 
-from mutagen._compat import PY3, text_type, PY2
+from mutagen._compat import PY3, text_type, PY2, izip
 from mutagen.asf import ASF, ASFHeaderError, ASFValue, UNICODE, DWORD, QWORD
 from mutagen.asf import BOOL, WORD, BYTEARRAY, GUID
+from mutagen.asf._util import guid2bytes, bytes2guid
+from mutagen.asf._objects import ContentDescriptionObject, \
+    ExtendedContentDescriptionObject, HeaderExtensionObject, \
+    MetadataObject, MetadataLibraryObject, CodecListObject, PaddingObject
 from mutagen.asf import ASFUnicodeAttribute, ASFError, ASFByteArrayAttribute, \
     ASFBoolAttribute, ASFDWordAttribute, ASFQWordAttribute, ASFWordAttribute, \
     ASFGUIDAttribute
@@ -24,6 +28,16 @@ class TASFFile(TestCase):
             os.path.join(DATA_DIR, "click.mpc"))
 
 
+class TASFMisc(TestCase):
+
+    def test_guid(self):
+        ex = "75B22633-668E-11CF-A6D9-00AA0062CE6C"
+        b = guid2bytes(ex)
+        self.assertEqual(len(b), 16)
+        self.assertTrue(isinstance(b, bytes))
+        self.assertEqual(bytes2guid(b), ex)
+
+
 class TASFInfo(TestCase):
 
     def setUp(self):
@@ -54,6 +68,34 @@ class TASFInfo(TestCase):
         self.failUnlessEqual(self.wma2.info.channels, 2)
         self.failUnlessEqual(self.wma3.info.channels, 2)
 
+    def test_codec_type(self):
+        self.assertEqual(self.wma1.info.codec_type,
+                         "Windows Media Audio 9 Standard")
+        self.assertEqual(self.wma2.info.codec_type,
+                         "Windows Media Audio 9 Professional")
+        self.assertEqual(self.wma3.info.codec_type,
+                         "Windows Media Audio 9 Lossless")
+
+    def test_codec_name(self):
+        self.assertEqual(self.wma1.info.codec_name,
+            "Windows Media Audio 9.1")
+        self.assertEqual(self.wma2.info.codec_name,
+            "Windows Media Audio 9.1 Professional")
+        self.assertEqual(self.wma3.info.codec_name,
+            "Windows Media Audio 9.1 Lossless")
+
+    def test_codec_description(self):
+        self.assertEqual(self.wma1.info.codec_description,
+            "64 kbps, 48 kHz, stereo 2-pass CBR")
+        self.assertEqual(self.wma2.info.codec_description,
+            "192 kbps, 44 kHz, 2 channel 24 bit 2-pass VBR")
+        self.assertEqual(self.wma3.info.codec_description,
+            "VBR Quality 100, 44 kHz, 2 channel 16 bit 1-pass VBR")
+
+    def test_pprint(self):
+        self.assertTrue(self.wma1.info.pprint())
+        self.assertTrue(isinstance(self.wma1.info.pprint(), text_type))
+
 
 class TASF(TestCase):
 
@@ -69,6 +111,18 @@ class TASF(TestCase):
 
 class TASFMixin(object):
 
+    def test_header_object_misc(self):
+        header = self.audio._header
+        header.pprint()
+        repr(header)
+
+    def test_delete(self):
+        self.audio["QL/Bla"] = u"Foooooooooooooooooo"
+        self.audio.save(padding=lambda x: 0)
+        filesize = os.path.getsize(self.audio.filename)
+        self.audio.delete()
+        self.assertTrue(os.path.getsize(self.audio.filename) < filesize)
+
     def test_pprint(self):
         self.failUnless(self.audio.pprint())
 
@@ -82,7 +136,7 @@ class TASFMixin(object):
         self.failUnless(key in self.audio.tags.as_dict().keys())
         newvalue = self.audio[key]
         if isinstance(newvalue, list):
-            for a, b in zip(sorted(newvalue), sorted(result or value)):
+            for a, b in izip(sorted(newvalue), sorted(result or value)):
                 self.failUnlessEqual(a, b)
         else:
             self.failUnlessEqual(self.audio[key], result or value)
@@ -534,24 +588,64 @@ class TASFLargeValue(TestCase):
         self.failIf("QL/GuidObject" not in dict(audio.to_metadata_library))
 
 
-class TASFUpdateSize(TestCase):
+class TASFSave(TestCase):
     # http://code.google.com/p/mutagen/issues/detail?id=81#c4
 
     original = os.path.join(DATA_DIR, "silence-1.wma")
 
     def setUp(self):
-        fd, self.filename = mkstemp(suffix='wma')
+        fd, self.filename = mkstemp(suffix='.wma')
         os.close(fd)
         shutil.copy(self.original, self.filename)
-        audio = ASF(self.filename)
-        audio["large_value1"] = "#" * 50000
-        audio.save()
+        self.audio = ASF(self.filename)
 
     def tearDown(self):
         os.unlink(self.filename)
 
+    def test_save_filename(self):
+        self.audio.save(self.audio.filename)
+
     def test_multiple_delete(self):
+        self.audio["large_value1"] = "#" * 50000
+        self.audio.save()
+
         audio = ASF(self.filename)
         for tag in audio.keys():
             del(audio[tag])
             audio.save()
+
+    def test_readd_objects(self):
+        header = self.audio._header
+        del header.objects[:]
+        self.audio.save()
+        self.assertTrue(header.get_child(ContentDescriptionObject.GUID))
+        self.assertTrue(
+            header.get_child(ExtendedContentDescriptionObject.GUID))
+        self.assertTrue(header.get_child(HeaderExtensionObject.GUID))
+        ext = header.get_child(HeaderExtensionObject.GUID)
+        self.assertTrue(ext.get_child(MetadataObject.GUID))
+        self.assertTrue(ext.get_child(MetadataLibraryObject.GUID))
+
+    def test_keep_others(self):
+        self.audio.save()
+        new = ASF(self.filename)
+        self.assertTrue(new._header.get_child(CodecListObject.GUID))
+
+    def test_padding(self):
+        old_tags = sorted(self.audio.items())
+
+        def get_padding(fn):
+            header = ASF(fn)._header
+            return len(header.get_child(PaddingObject.GUID).data)
+
+        for i in [0, 1, 2, 3, 42, 100, 5000, 30432, 1]:
+
+            def padding_cb(info):
+                self.assertEqual(info.size, 30432)
+                return i
+
+            self.audio.save(padding=padding_cb)
+            self.assertEqual(get_padding(self.filename), i)
+
+        last = ASF(self.filename)
+        self.assertEqual(sorted(last.items()), old_tags)
