@@ -1,7 +1,9 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 import pickle
-from tests import add, TestCase
+from tests import TestCase, DATA_DIR
 from mutagen.id3 import ID3FileType, ID3
 from mutagen.easyid3 import EasyID3, error as ID3Error
 from mutagen._compat import PY3
@@ -13,12 +15,12 @@ class TEasyID3(TestCase):
     def setUp(self):
         fd, self.filename = mkstemp('.mp3')
         os.close(fd)
-        empty = os.path.join('tests', 'data', 'emptyfile.mp3')
+        empty = os.path.join(DATA_DIR, 'emptyfile.mp3')
         shutil.copy(empty, self.filename)
         self.id3 = EasyID3()
 
     def test_remember_ctr(self):
-        empty = os.path.join('tests', 'data', 'emptyfile.mp3')
+        empty = os.path.join(DATA_DIR, 'emptyfile.mp3')
         mp3 = ID3FileType(empty, ID3=EasyID3)
         self.failIf(mp3.tags)
         mp3["artist"] = ["testing"]
@@ -51,16 +53,16 @@ class TEasyID3(TestCase):
             self.failIf(self.id3.has_key("foo"))
 
     def test_empty_file(self):
-        empty = os.path.join('tests', 'data', 'emptyfile.mp3')
+        empty = os.path.join(DATA_DIR, 'emptyfile.mp3')
         self.assertRaises(ID3Error, EasyID3, filename=empty)
 
     def test_nonexistent_file(self):
-        empty = os.path.join('tests', 'data', 'does', 'not', 'exist')
+        empty = os.path.join(DATA_DIR, 'does', 'not', 'exist')
         self.assertRaises(IOError, EasyID3, filename=empty)
 
     def test_write_single(self):
         for key in EasyID3.valid_keys:
-            if key == "date":
+            if (key == "date") or (key == "originaldate"):
                 continue
             elif key.startswith("replaygain_"):
                 continue
@@ -83,7 +85,7 @@ class TEasyID3(TestCase):
 
     def test_write_double(self):
         for key in EasyID3.valid_keys:
-            if key == "date":
+            if (key == "date") or (key == "originaldate"):
                 continue
             elif key.startswith("replaygain_"):
                 continue
@@ -134,6 +136,34 @@ class TEasyID3(TestCase):
         id3 = EasyID3(self.filename)
         self.failUnlessEqual(id3["date"], ["2004", "2005"])
 
+    def test_write_original_date(self):
+        self.id3["originaldate"] = "2004"
+        self.id3.save(self.filename)
+        id3 = EasyID3(self.filename)
+        self.failUnlessEqual(id3["originaldate"], ["2004"])
+
+        self.id3["originaldate"] = "2004"
+        self.id3.save(self.filename)
+        id3 = EasyID3(self.filename)
+        self.failUnlessEqual(id3["originaldate"], ["2004"])
+
+    def test_original_date_delete(self):
+        self.id3["originaldate"] = "2004"
+        self.failUnlessEqual(self.id3["originaldate"], ["2004"])
+        del(self.id3["originaldate"])
+        self.failIf("originaldate" in self.id3.keys())
+
+    def test_write_original_date_double(self):
+        self.id3["originaldate"] = ["2004", "2005"]
+        self.id3.save(self.filename)
+        id3 = EasyID3(self.filename)
+        self.failUnlessEqual(id3["originaldate"], ["2004", "2005"])
+
+        self.id3["originaldate"] = ["2004", "2005"]
+        self.id3.save(self.filename)
+        id3 = EasyID3(self.filename)
+        self.failUnlessEqual(id3["originaldate"], ["2004", "2005"])
+
     def test_write_invalid(self):
         self.failUnlessRaises(ValueError, self.id3.__getitem__, "notvalid")
         self.failUnlessRaises(ValueError, self.id3.__delitem__, "notvalid")
@@ -319,5 +349,3 @@ class TEasyID3(TestCase):
 
     def tearDown(self):
         os.unlink(self.filename)
-
-add(TEasyID3)
