@@ -59,22 +59,35 @@ class TMetadataBlock(TestCase):
         b.data = b"quux"
         self.failUnlessEqual(b.write(), b"quux")
 
-    def test_writeblocks(self):
-        blocks = [Padding(b"\x00" * 20), Padding(b"\x00" * 30)]
-        self.failUnlessEqual(len(MetadataBlock.writeblocks(blocks)), 58)
+    def test_write_read_max_size(self):
+
+        class SomeBlock(MetadataBlock):
+            code = 255
+
+        max_data_size = 2 ** 24 - 1
+        block = SomeBlock(b"\x00" * max_data_size)
+        data = MetadataBlock._writeblock(block)
+        self.assertEqual(data[:4], b"\xff\xff\xff\xff")
+        header_size = 4
+        self.assertEqual(len(data), max_data_size + header_size)
+
+        block = SomeBlock(b"\x00" * (max_data_size + 1))
+        self.assertRaises(error, MetadataBlock._writeblock, block)
 
     def test_ctr_garbage(self):
         self.failUnlessRaises(TypeError, StreamInfo, 12)
 
-    def test_group_padding(self):
-        blocks = [Padding(b"\x00" * 20), Padding(b"\x00" * 30),
-                  MetadataBlock(b"foobar")]
-        blocks[-1].code = 0
-        length1 = len(MetadataBlock.writeblocks(blocks))
-        MetadataBlock.group_padding(blocks)
-        length2 = len(MetadataBlock.writeblocks(blocks))
-        self.failUnlessEqual(length1, length2)
-        self.failUnlessEqual(len(blocks), 2)
+    def test_too_large(self):
+        block = Picture()
+        block.data = b"\x00" * 0x1FFFFFF
+        self.assertRaises(
+            error, MetadataBlock._writeblocks, [block], 0, 0, None)
+
+    def test_too_large_padding(self):
+        block = Padding()
+        self.assertEqual(
+            len(MetadataBlock._writeblocks([block], 0, 0, lambda x: 2 ** 24)),
+            2**24 - 1 + 4)
 
 
 class TStreamInfo(TestCase):
@@ -278,12 +291,52 @@ class TFLAC(TestCase):
     def tearDown(self):
         os.unlink(self.NEW)
 
+    def test_padding(self):
+        for pad in [0, 42, 2**24 - 1, 2 ** 24]:
+            self.flac.save(padding=lambda x: pad)
+            new = FLAC(self.flac.filename)
+            expected = min(2**24 - 1, pad)
+            self.assertEqual(new.metadata_blocks[-1].length, expected)
+
+    def test_save_multiple_padding(self):
+        # we don't touch existing padding blocks on save, but will
+        # replace them in the file with one at the end
+
+        def num_padding(f):
+            blocks = f.metadata_blocks
+            return len([b for b in blocks if isinstance(b, Padding)])
+
+        num_blocks = num_padding(self.flac)
+        self.assertEqual(num_blocks, 1)
+        block = Padding()
+        block.length = 42
+        self.flac.metadata_blocks.append(block)
+        block = Padding()
+        block.length = 24
+        self.flac.metadata_blocks.append(block)
+        self.flac.save()
+        self.assertEqual(num_padding(self.flac), num_blocks + 2)
+
+        new = FLAC(self.flac.filename)
+        self.assertEqual(num_padding(new), 1)
+        self.assertTrue(isinstance(new.metadata_blocks[-1], Padding))
+
+    def test_increase_size_new_padding(self):
+        self.assertEqual(self.flac.metadata_blocks[-1].length, 3060)
+        value = u"foo" * 100
+        self.flac[u"foo"] = [value]
+        self.flac.save()
+        new = FLAC(self.NEW)
+        self.assertEqual(new.metadata_blocks[-1].length, 2752)
+        self.assertEqual(new[u"foo"], [value])
+
     def test_delete(self):
         self.failUnless(self.flac.tags)
         self.flac.delete()
-        self.failIf(self.flac.tags)
+        self.assertTrue(self.flac.tags is not None)
+        self.assertFalse(self.flac.tags)
         flac = FLAC(self.NEW)
-        self.failIf(flac.tags)
+        self.assertTrue(flac.tags is None)
 
     def test_module_delete(self):
         delete(self.NEW)
@@ -483,6 +536,10 @@ class TFLAC(TestCase):
         f = FLAC(self.NEW)
         self.failUnlessEqual(f['title'], ['vc title'])
 
+    def test_save_on_mp3(self):
+        path = os.path.join(DATA_DIR, "silence-44-s.mp3")
+        self.assertRaises(error, self.flac.save, path)
+
     def test_mime(self):
         self.failUnless("audio/x-flac" in self.flac.mime)
 
@@ -558,15 +615,15 @@ class TFLACBadBlockSizeOverflow(TestCase):
     def test_largest_valid(self):
         f = FLAC(self.filename)
         pic = Picture()
-        pic.data = b"\x00" * (2 ** 24 - 32)
-        self.assertEqual(len(pic.write()), 2 ** 24)
+        pic.data = b"\x00" * (2 ** 24 - 1 - 32)
+        self.assertEqual(len(pic.write()), 2 ** 24 - 1)
         f.add_picture(pic)
         f.save()
 
     def test_smallest_invalid(self):
         f = FLAC(self.filename)
         pic = Picture()
-        pic.data = b"\x00" * (2 ** 24 - 31)
+        pic.data = b"\x00" * (2 ** 24 - 32)
         f.add_picture(pic)
         self.assertRaises(error, f.save)
 
@@ -576,7 +633,7 @@ class TFLACBadBlockSizeOverflow(TestCase):
         f = FLAC(self.filename)
         f.clear_pictures()
         pic = Picture()
-        pic.data = b"\x00" * (2 ** 24 - 31)
+        pic.data = b"\x00" * (2 ** 24 - 32)
         pic._invalid_overflow_size = 42
         f.add_picture(pic)
         f.save()
@@ -584,7 +641,7 @@ class TFLACBadBlockSizeOverflow(TestCase):
         # make sure we can read it and save it again
         f = FLAC(self.filename)
         self.assertTrue(f.pictures)
-        self.assertEqual(len(f.pictures[0].data), 2 ** 24 - 31)
+        self.assertEqual(len(f.pictures[0].data), 2 ** 24 - 32)
         f.save()
 
 
@@ -655,15 +712,6 @@ class CVE20074619(TestCase):
         f.metadata_blocks[-1].length = 0xFFFFFFFFFFFFFFFF
         self.failUnlessRaises(IOError, f.metadata_blocks[-1].write)
 
-    def test_12_write_too_big_for_flac(self):
-        from mutagen.flac import MetadataBlock
-        filename = os.path.join(DATA_DIR, "silence-44-s.flac")
-        f = FLAC(filename)
-        # This size is too big to be in a FLAC block but is overwise fine.
-        f.metadata_blocks[-1].length = 0x1FFFFFF
-        self.failUnlessRaises(
-            IOError, MetadataBlock.writeblocks, [f.metadata_blocks[-1]])
-
     # Vulnerability 13 and 14 are specific to libFLAC and C/C++ memory
     # management schemes.
 
