@@ -9,7 +9,7 @@ from mutagen.apev2 import APEv2
 from mutagen.id3 import ID3, COMR, Frames, Frames_2_2, ID3Warning, \
     ID3JunkFrameError, ID3Header, ID3UnsupportedVersionError, _fullread
 from mutagen.id3._util import BitPaddedInt, error as ID3Error
-from mutagen._compat import cBytesIO, PY2, iteritems, integer_types
+from mutagen._compat import cBytesIO, PY2, iteritems, integer_types, izip
 import warnings
 from tempfile import mkstemp
 warnings.simplefilter('error', ID3Warning)
@@ -90,6 +90,10 @@ class ID3Loading(TestCase):
         name = join(DATA_DIR, 'does', 'not', 'exist')
         self.assertRaises(EnvironmentError, ID3, name)
 
+    def test_read_padding(self):
+        self.assertEqual(ID3(self.silence)._padding, 1142)
+        self.assertEqual(ID3(self.unsynch)._padding, 0)
+
     def test_header_empty(self):
         fileobj = open(self.empty, 'rb')
         self.assertRaises(ID3Error, ID3Header, fileobj)
@@ -873,7 +877,7 @@ def create_read_tag_tests():
                 if not isinstance(value, list):
                     value = [value]
                     t = [t]
-                for value, t in zip(value, iter(t)):
+                for value, t in izip(value, iter(t)):
                     if isinstance(value, float):
                         self.failUnlessAlmostEqual(value, getattr(t, attr), 5)
                     else:
@@ -1290,11 +1294,6 @@ class WriteRoundtrip(TestCase):
         id3 = ID3(self.newsilence)
         self.assertEquals(id3["TPE2"], "Ab" * 1025)
 
-    def test_nofile_emptytag(self):
-        os.unlink(self.newsilence)
-        ID3().save(self.newsilence)
-        self.assertRaises(EnvironmentError, open, self.newsilence)
-
     def test_nofile_silencetag(self):
         id3 = ID3(self.newsilence)
         os.unlink(self.newsilence)
@@ -1317,14 +1316,6 @@ class WriteRoundtrip(TestCase):
         self.failIf(id3)
         self.assertEquals(open(self.newsilence, 'rb').read(10), b'')
 
-    def test_empty_plustag_emptytag_empty(self):
-        id3 = ID3(self.newsilence)
-        open(self.newsilence, 'wb').truncate()
-        id3.save()
-        id3.clear()
-        id3.save()
-        self.assertEquals(open(self.newsilence, 'rb').read(10), b'')
-
     def test_delete_invalid_zero(self):
         f = open(self.newsilence, 'wb')
         f.write(b'ID3\x04\x00\x00\x00\x00\x00\x00abc')
@@ -1753,6 +1744,41 @@ class TID3Misc(TestCase):
         self.assertTrue(determine_bpi(d, Frames) is BitPaddedInt)
 
 
+class TID3Padding(TestCase):
+
+    def setUp(self):
+        fd, self.filename = mkstemp(suffix='.mp3')
+        os.close(fd)
+        orig = os.path.join(DATA_DIR, "silence-44-s.mp3")
+        shutil.copy(orig, self.filename)
+
+    def tearDown(self):
+        os.remove(self.filename)
+
+    def test_fill_all(self):
+        tag = ID3(self.filename)
+        self.assertEqual(tag._padding, 1142)
+        tag.delall("TPE1")
+        # saving should increase the padding not decrease the tag size
+        tag.save()
+        tag = ID3(self.filename)
+        self.assertEqual(tag._padding, 1166)
+
+    def test_remove_add_padding(self):
+        ID3(self.filename).save()
+
+        tag = ID3(self.filename)
+        old_padding = tag._padding
+        old_size = os.path.getsize(self.filename)
+        tag.save(padding=lambda x: 0)
+        self.assertEqual(os.path.getsize(self.filename),
+                         old_size - old_padding)
+        old_size = old_size - old_padding
+        tag.save(padding=lambda x: 137)
+        self.assertEqual(os.path.getsize(self.filename),
+                         old_size + 137)
+
+
 class TID3Corrupt(TestCase):
 
     def setUp(self):
@@ -1774,6 +1800,12 @@ class TID3Corrupt(TestCase):
             h.truncate(50)
         self.assertRaises(id3.error, ID3, self.filename)
 
+    def test_save(self):
+        with open(self.filename, "r+b") as h:
+            h.seek(5, 0)
+            h.write(b"nope")
+        self.assertRaises(id3.error, ID3().save, self.filename)
+
 try:
     import eyeD3
 except ImportError:
