@@ -5,7 +5,7 @@ import shutil
 import struct
 import subprocess
 
-from mutagen._compat import cBytesIO, PY3, text_type, PY2
+from mutagen._compat import cBytesIO, PY3, text_type, PY2, izip
 from tempfile import mkstemp
 from tests import TestCase, DATA_DIR
 from mutagen.mp4 import (MP4, Atom, Atoms, MP4Tags, MP4Info, delete, MP4Cover,
@@ -26,7 +26,9 @@ class TAtom(TestCase):
     def test_length_1(self):
         fileobj = cBytesIO(b"\x00\x00\x00\x01atom"
                            b"\x00\x00\x00\x00\x00\x00\x00\x10" + b"\x00" * 16)
-        self.failUnlessEqual(Atom(fileobj).length, 16)
+        atom = Atom(fileobj)
+        self.failUnlessEqual(atom.length, 16)
+        self.failUnlessEqual(atom.datalength, 0)
 
     def test_length_64bit_less_than_16(self):
         fileobj = cBytesIO(b"\x00\x00\x00\x01atom"
@@ -64,6 +66,7 @@ class TAtom(TestCase):
         atom = Atom(fileobj)
         self.failUnlessEqual(fileobj.tell(), 48)
         self.failUnlessEqual(atom.length, 48)
+        self.failUnlessEqual(atom.datalength, 40)
 
     def test_length_0_container(self):
         data = cBytesIO(struct.pack(">I4s", 0, b"moov") +
@@ -320,27 +323,31 @@ class TMP4Tags(TestCase):
         fd, filename = mkstemp(suffix='.mp4')
         os.close(fd)
         shutil.copy(original, filename)
-        delete(filename)
+        try:
+            delete(filename)
 
-        # it should still end up in the file
-        tags.save(filename)
-        with open(filename, "rb") as h:
-            self.assertTrue(b"wheeee" in h.read())
+            # it should still end up in the file
+            tags.save(filename)
+            with open(filename, "rb") as h:
+                self.assertTrue(b"wheeee" in h.read())
 
-        # if we define our own aART throw away the broken one
-        tags["aART"] = ["new"]
-        tags.save(filename)
-        with open(filename, "rb") as h:
-            self.assertFalse(b"wheeee" in h.read())
+            # if we define our own aART throw away the broken one
+            tags["aART"] = ["new"]
+            tags.save(filename)
+            with open(filename, "rb") as h:
+                self.assertFalse(b"wheeee" in h.read())
 
-        # add the broken one back and delete all tags including the broken one
-        del tags["aART"]
-        tags.save(filename)
-        with open(filename, "rb") as h:
-            self.assertTrue(b"wheeee" in h.read())
-        delete(filename)
-        with open(filename, "rb") as h:
-            self.assertFalse(b"wheeee" in h.read())
+            # add the broken one back and delete all tags including
+            # the broken one
+            del tags["aART"]
+            tags.save(filename)
+            with open(filename, "rb") as h:
+                self.assertTrue(b"wheeee" in h.read())
+            delete(filename)
+            with open(filename, "rb") as h:
+                self.assertFalse(b"wheeee" in h.read())
+        finally:
+            os.unlink(filename)
 
     def test_render_freeform(self):
         data = (
@@ -483,35 +490,37 @@ class TMP4Mixin(object):
     def test_padding_2(self):
         self.audio["\xa9nam"] = u"wheeee" * 10
         self.audio.save()
+
         # Reorder "free" and "ilst" atoms
-        fileobj = open(self.audio.filename, "rb+")
-        atoms = Atoms(fileobj)
-        meta = atoms[b"moov", b"udta", b"meta"]
-        meta_length1 = meta.length
-        ilst = meta[b"ilst", ]
-        free = meta[b"free", ]
-        self.failUnlessEqual(ilst.offset + ilst.length, free.offset)
-        fileobj.seek(ilst.offset)
-        ilst_data = fileobj.read(ilst.length)
-        fileobj.seek(free.offset)
-        free_data = fileobj.read(free.length)
-        fileobj.seek(ilst.offset)
-        fileobj.write(free_data + ilst_data)
-        fileobj.close()
-        fileobj = open(self.audio.filename, "rb+")
-        atoms = Atoms(fileobj)
-        meta = atoms[b"moov", b"udta", b"meta"]
-        ilst = meta[b"ilst", ]
-        free = meta[b"free", ]
-        self.failUnlessEqual(free.offset + free.length, ilst.offset)
-        fileobj.close()
+        with open(self.audio.filename, "rb+") as fileobj:
+            atoms = Atoms(fileobj)
+            meta = atoms[b"moov", b"udta", b"meta"]
+            meta_length1 = meta.length
+            ilst = meta[b"ilst", ]
+            free = meta[b"free", ]
+            self.failUnlessEqual(ilst.offset + ilst.length, free.offset)
+            fileobj.seek(ilst.offset)
+            ilst_data = fileobj.read(ilst.length)
+            fileobj.seek(free.offset)
+            free_data = fileobj.read(free.length)
+            fileobj.seek(ilst.offset)
+            fileobj.write(free_data + ilst_data)
+
+        with open(self.audio.filename, "rb+") as fileobj:
+            atoms = Atoms(fileobj)
+            meta = atoms[b"moov", b"udta", b"meta"]
+            ilst = meta[b"ilst", ]
+            free = meta[b"free", ]
+            self.failUnlessEqual(free.offset + free.length, ilst.offset)
+
         # Save the file
         self.audio["\xa9nam"] = u"wheeee" * 11
         self.audio.save()
+
         # Check the order of "free" and "ilst" atoms
-        fileobj = open(self.audio.filename, "rb+")
-        atoms = Atoms(fileobj)
-        fileobj.close()
+        with open(self.audio.filename, "rb+") as fileobj:
+            atoms = Atoms(fileobj)
+
         meta = atoms[b"moov", b"udta", b"meta"]
         ilst = meta[b"ilst", ]
         free = meta[b"free", ]
@@ -708,12 +717,32 @@ class TMP4Mixin(object):
         self.audio["\xa9nam"] = "wheeeeeeee"
         self.audio.save()
         bb = self.__read_offsets(self.filename)
-        for a, b in zip(aa, bb):
+        for a, b in izip(aa, bb):
             self.failUnlessEqual(a, b)
 
     def test_mime(self):
         self.failUnless("audio/mp4" in self.audio.mime)
 
+    def test_set_init_padding_zero(self):
+        if self.audio.tags is None:
+            self.audio.add_tags()
+        self.audio.save(padding=lambda x: 0)
+        self.assertEqual(MP4(self.audio.filename)._padding, 0)
+
+    def test_set_init_padding_large(self):
+        if self.audio.tags is None:
+            self.audio.add_tags()
+        self.audio.save(padding=lambda x: 5000)
+        self.assertEqual(MP4(self.audio.filename)._padding, 5000)
+
+    def test_set_various_padding(self):
+        if self.audio.tags is None:
+            self.audio.add_tags()
+        for i in [0, 1, 2, 3, 1024, 983, 5000, 0, 1]:
+            self.audio.save(padding=lambda x: i)
+            self.assertEqual(MP4(self.audio.filename)._padding, i)
+            self.faad()
+
 
 class TMP4HasTagsMixin(TMP4Mixin):
     def test_save_simple(self):
@@ -745,6 +774,14 @@ class TMP4HasTagsMixin(TMP4Mixin):
         self.failUnlessRaisesRegexp(
             error, "MP4", MP4, os.path.join(DATA_DIR, "empty.ogg"))
 
+    def test_delete_remove_padding(self):
+        self.audio.clear()
+        self.audio.tags['foob'] = u"foo"
+        self.audio.save(padding=lambda x: 0)
+        filesize = os.path.getsize(self.audio.filename)
+        self.audio.delete()
+        self.assertTrue(os.path.getsize(self.audio.filename) < filesize)
+
 
 class TMP4Datatypes(TMP4, TMP4HasTagsMixin):
     original = os.path.join(DATA_DIR, "has-tags.m4a")
@@ -767,6 +804,9 @@ class TMP4Datatypes(TMP4, TMP4HasTagsMixin):
         text = self.audio.tags.pprint().splitlines()
         self.assertTrue(u"©ART=Test Artist" in text)
 
+    def test_get_padding(self):
+        self.assertEqual(self.audio._padding, 1634)
+
 
 class TMP4CovrWithName(TMP4, TMP4Mixin):
     # http://bugs.musicbrainz.org/ticket/5894
@@ -839,13 +879,13 @@ class TMP4UpdateParents64Bit(TestCase):
             self.assertEqual(61, atoms.atoms[0].children[0].length)
             tags = MP4Tags(atoms, fileobj)
             tags['pgap'] = True
-            tags.save(self.filename)
+            tags.save(self.filename, padding=lambda x: 0)
 
         with open(self.filename, "rb") as fileobj:
             atoms = Atoms(fileobj)
             # original size + 'pgap' size + padding
-            self.assertEqual(77 + 25 + 974, atoms.atoms[0].length)
-            self.assertEqual(61 + 25 + 974, atoms.atoms[0].children[0].length)
+            self.assertEqual(77 + 25 + 8, atoms.atoms[0].length)
+            self.assertEqual(61 + 25 + 8, atoms.atoms[0].children[0].length)
 
     def tearDown(self):
         os.unlink(self.filename)
