@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 from tempfile import mkstemp
@@ -5,21 +7,21 @@ from tempfile import mkstemp
 from mutagen.id3 import ID3, TIT2
 from mutagen.musepack import Musepack, MusepackInfo, MusepackHeaderError
 from mutagen._compat import cBytesIO
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 
 
 class TMusepack(TestCase):
 
     def setUp(self):
-        self.sv8 = Musepack(os.path.join("tests", "data", "sv8_header.mpc"))
-        self.sv7 = Musepack(os.path.join("tests", "data", "click.mpc"))
-        self.sv5 = Musepack(os.path.join("tests", "data", "sv5_header.mpc"))
-        self.sv4 = Musepack(os.path.join("tests", "data", "sv4_header.mpc"))
+        self.sv8 = Musepack(os.path.join(DATA_DIR, "sv8_header.mpc"))
+        self.sv7 = Musepack(os.path.join(DATA_DIR, "click.mpc"))
+        self.sv5 = Musepack(os.path.join(DATA_DIR, "sv5_header.mpc"))
+        self.sv4 = Musepack(os.path.join(DATA_DIR, "sv4_header.mpc"))
 
     def test_bad_header(self):
         self.failUnlessRaises(
             MusepackHeaderError,
-            Musepack, os.path.join("tests", "data", "almostempty.mpc"))
+            Musepack, os.path.join(DATA_DIR, "almostempty.mpc"))
 
     def test_channels(self):
         self.failUnlessEqual(self.sv8.info.channels, 2)
@@ -63,10 +65,10 @@ class TMusepack(TestCase):
     def test_not_my_file(self):
         self.failUnlessRaises(
             MusepackHeaderError, Musepack,
-            os.path.join("tests", "data", "empty.ogg"))
+            os.path.join(DATA_DIR, "empty.ogg"))
         self.failUnlessRaises(
             MusepackHeaderError, Musepack,
-            os.path.join("tests", "data", "emptyfile.mp3"))
+            os.path.join(DATA_DIR, "emptyfile.mp3"))
 
     def test_almost_my_file(self):
         self.failUnlessRaises(
@@ -89,11 +91,19 @@ class TMusepack(TestCase):
     def test_mime(self):
         self.failUnless("audio/x-musepack" in self.sv7.mime)
 
-add(TMusepack)
+    def test_zero_padded_sh_packet(self):
+        # https://bitbucket.org/lazka/mutagen/issue/198
+        data = (b"MPCKSH\x10\x95 Q\xa2\x08\x81\xb8\xc9T\x00\x1e\x1b"
+                b"\x00RG\x0c\x01A\xcdY\x06?\x80Z\x06EI")
+
+        fileobj = cBytesIO(data)
+        info = MusepackInfo(fileobj)
+        self.assertEqual(info.channels, 2)
+        self.assertEqual(info.samples, 3024084)
 
 
 class TMusepackWithID3(TestCase):
-    SAMPLE = os.path.join("tests", "data", "click.mpc")
+    SAMPLE = os.path.join(DATA_DIR, "click.mpc")
 
     def setUp(self):
         fd, self.NEW = mkstemp(suffix='mpc')
@@ -117,5 +127,3 @@ class TMusepackWithID3(TestCase):
         self.failUnlessEqual(id3['TIT2'], 'id3 title')
         f = Musepack(self.NEW)
         self.failUnlessEqual(f['title'], 'apev2 title')
-
-add(TMusepackWithID3)
