@@ -5,10 +5,11 @@ import random
 import shutil
 import subprocess
 
-from mutagen._compat import BytesIO
+from mutagen._compat import BytesIO, xrange
 from tests import TestCase, DATA_DIR
 from mutagen.ogg import OggPage, error as OggError
 from mutagen._util import cdata
+from mutagen import _util
 from tempfile import mkstemp
 
 
@@ -22,7 +23,7 @@ class TOggPage(TestCase):
         pages[0].packets = [b"foo"]
         pages[1].packets = [b"bar"]
         pages[2].packets = [b"baz"]
-        for i in range(len(pages)):
+        for i in xrange(len(pages)):
             pages[i].sequence = i
         for page in pages:
             page.serial = 1
@@ -88,6 +89,119 @@ class TOggPage(TestCase):
         self.failUnlessEqual(len(pages), 2)
         self.failUnlessEqual(OggPage.to_packets(pages), packets)
 
+    def test_replace(self):
+        # create interleaved pages
+        fileobj = BytesIO()
+        pages = [OggPage(), OggPage(), OggPage()]
+        pages[0].serial = 42
+        pages[0].sequence = 0
+        pages[0].packets = [b"foo"]
+        pages[1].serial = 24
+        pages[1].sequence = 0
+        pages[1].packets = [b"bar"]
+        pages[2].serial = 42
+        pages[2].sequence = 1
+        pages[2].packets = [b"baz"]
+        for page in pages:
+            fileobj.write(page.write())
+
+        fileobj.seek(0, 0)
+        pages_from_file = [OggPage(fileobj), OggPage(fileobj),
+                           OggPage(fileobj)]
+
+        old_pages = [pages_from_file[0], pages_from_file[2]]
+        packets = OggPage.to_packets(old_pages, strict=True)
+        self.assertEqual(packets, [b"foo", b"baz"])
+        new_packets = [b"1111", b"2222"]
+        new_pages = OggPage.from_packets(new_packets,
+                                         sequence=old_pages[0].sequence)
+        self.assertEqual(len(new_pages), 1)
+        OggPage.replace(fileobj, old_pages, new_pages)
+
+        fileobj.seek(0, 0)
+        first = OggPage(fileobj)
+        self.assertEqual(first.serial, 42)
+        self.assertEqual(OggPage.to_packets([first], strict=True),
+                         [b"1111", b"2222"])
+        second = OggPage(fileobj)
+        self.assertEqual(second.serial, 24)
+        self.assertEqual(OggPage.to_packets([second], strict=True), [b"bar"])
+
+    def test_replace_fast_path(self):
+        # create interleaved pages
+        fileobj = BytesIO()
+        pages = [OggPage(), OggPage(), OggPage()]
+        pages[0].serial = 42
+        pages[0].sequence = 0
+        pages[0].packets = [b"foo"]
+        pages[1].serial = 24
+        pages[1].sequence = 0
+        pages[1].packets = [b"bar"]
+        pages[2].serial = 42
+        pages[2].sequence = 1
+        pages[2].packets = [b"baz"]
+        for page in pages:
+            fileobj.write(page.write())
+
+        fileobj.seek(0, 0)
+        pages_from_file = [OggPage(fileobj), OggPage(fileobj),
+                           OggPage(fileobj)]
+
+        old_pages = [pages_from_file[0], pages_from_file[2]]
+        packets = OggPage.to_packets(old_pages, strict=True)
+        self.assertEqual(packets, [b"foo", b"baz"])
+        new_packets = [b"111", b"222"]
+        new_pages = OggPage._from_packets_try_preserve(new_packets, old_pages)
+        self.assertEqual(len(new_pages), 2)
+
+        # remove insert_bytes, so we can be sure the fast path was taken
+        old_insert_bytes = _util.insert_bytes
+        _util.insert_bytes = None
+        try:
+            OggPage.replace(fileobj, old_pages, new_pages)
+        finally:
+            _util.insert_bytes = old_insert_bytes
+
+        # validate that the new data was written and the other pages
+        # are untouched
+        fileobj.seek(0, 0)
+        pages_from_file = [OggPage(fileobj), OggPage(fileobj),
+                           OggPage(fileobj)]
+        packets = OggPage.to_packets(
+            [pages_from_file[0], pages_from_file[2]], strict=True)
+        self.assertEqual(packets, [b"111", b"222"])
+        packets = OggPage.to_packets([pages_from_file[1]], strict=True)
+        self.assertEqual(packets, [b"bar"])
+
+    def test_replace_continued(self):
+        # take a partial packet and replace it with a new page
+        # replace() should make it spanning again
+        fileobj = BytesIO()
+        pages = [OggPage(), OggPage()]
+        pages[0].serial = 1
+        pages[0].sequence = 0
+        pages[0].complete = False
+        pages[0].packets = [b"foo"]
+        pages[1].serial = 1
+        pages[1].sequence = 1
+        pages[1].continued = True
+        pages[1].packets = [b"bar"]
+        fileobj = BytesIO()
+        for page in pages:
+            fileobj.write(page.write())
+
+        fileobj.seek(0, 0)
+        pages_from_file = [OggPage(fileobj), OggPage(fileobj)]
+        self.assertEqual(OggPage.to_packets(pages_from_file), [b"foobar"])
+        packets_part = OggPage.to_packets([pages_from_file[0]])
+        self.assertEqual(packets_part, [b"foo"])
+        new_pages = OggPage.from_packets([b"quuux"])
+        OggPage.replace(fileobj, [pages_from_file[0]], new_pages)
+
+        fileobj.seek(0, 0)
+        written = OggPage.to_packets([OggPage(fileobj), OggPage(fileobj)])
+        self.assertEquals(written, [b"quuuxbar"])
+
     def test_renumber(self):
         self.failUnlessEqual(
             [page.sequence for page in self.pages], [0, 1, 2])
@@ -97,13 +211,13 @@ class TOggPage(TestCase):
         fileobj.seek(0)
         OggPage.renumber(fileobj, 1, 10)
         fileobj.seek(0)
-        pages = [OggPage(fileobj) for i in range(3)]
+        pages = [OggPage(fileobj) for i in xrange(3)]
         self.failUnlessEqual([page.sequence for page in pages], [10, 11, 12])
 
         fileobj.seek(0)
         OggPage.renumber(fileobj, 1, 20)
         fileobj.seek(0)
-        pages = [OggPage(fileobj) for i in range(3)]
+        pages = [OggPage(fileobj) for i in xrange(3)]
         self.failUnlessEqual([page.sequence for page in pages], [20, 21, 22])
 
     def test_renumber_extradata(self):
@@ -116,7 +230,7 @@ class TOggPage(TestCase):
         self.failUnlessRaises(Exception, OggPage.renumber, fileobj, 1, 10)
         fileobj.seek(0)
         # But the already written data should remain valid,
-        pages = [OggPage(fileobj) for i in range(3)]
+        pages = [OggPage(fileobj) for i in xrange(3)]
         self.failUnlessEqual([page.sequence for page in pages], [10, 11, 12])
         # And the garbage that caused the error should be okay too.
         self.failUnlessEqual(fileobj.read(), b"left over data")
@@ -127,12 +241,10 @@ class TOggPage(TestCase):
             os.close(fd)
             shutil.copy(os.path.join(DATA_DIR, "multipagecomment.ogg"),
                         filename)
-            fileobj = open(filename, "rb+")
-            OggPage.renumber(fileobj, 1002429366, 20)
-            fileobj.close()
-            fileobj = open(filename, "rb+")
-            OggPage.renumber(fileobj, 1002429366, 0)
-            fileobj.close()
+            with open(filename, "rb+") as fileobj:
+                OggPage.renumber(fileobj, 1002429366, 20)
+            with open(filename, "rb+") as fileobj:
+                OggPage.renumber(fileobj, 1002429366, 0)
         finally:
             try:
                 os.unlink(filename)
@@ -140,7 +252,7 @@ class TOggPage(TestCase):
                 pass
 
     def test_renumber_muxed(self):
-        pages = [OggPage() for i in range(10)]
+        pages = [OggPage() for i in xrange(10)]
         for seq, page in enumerate(pages[0:1] + pages[2:]):
             page.serial = 0
             page.sequence = seq
@@ -149,12 +261,12 @@ class TOggPage(TestCase):
         data = BytesIO(b"".join([page.write() for page in pages]))
         OggPage.renumber(data, 0, 20)
         data.seek(0)
-        pages = [OggPage(data) for i in range(10)]
+        pages = [OggPage(data) for i in xrange(10)]
         self.failUnlessEqual(pages[1].serial, 2)
         self.failUnlessEqual(pages[1].sequence, 100)
         pages.pop(1)
         self.failUnlessEqual(
-            [page.sequence for page in pages], list(range(20, 29)))
+            [page.sequence for page in pages], list(xrange(20, 29)))
 
     def test_to_packets(self):
         self.failUnlessEqual(
@@ -208,16 +320,36 @@ class TOggPage(TestCase):
         self.failUnless(pages[1].continued)
         self.failUnlessEqual(OggPage.to_packets(pages), packets)
 
+    def test__from_packets_try_preserve(self):
+        # if the packet layout matches, just create pages with
+        # the same layout and copy things over
+        packets = [b"1" * 100000, b"2" * 100000, b"3" * 100000]
+        pages = OggPage.from_packets(packets, sequence=42, default_size=977)
+        new_pages = OggPage._from_packets_try_preserve(packets, pages)
+        self.assertEqual(pages, new_pages)
+
+        # zero case
+        new_pages = OggPage._from_packets_try_preserve([], pages)
+        self.assertEqual(new_pages, [])
+
+        # if the layout doesn't match we should fall back to creating new
+        # pages starting with the sequence of the first given page
+        other_packets = list(packets)
+        other_packets[1] += b"\xff"
+        other_pages = OggPage.from_packets(other_packets, 42)
+        new_pages = OggPage._from_packets_try_preserve(other_packets, pages)
+        self.assertEqual(new_pages, other_pages)
+
     def test_random_data_roundtrip(self):
         try:
             random_file = open("/dev/urandom", "rb")
         except (IOError, OSError):
             print("WARNING: Random data round trip test disabled.")
             return
-        for i in range(10):
+        for i in xrange(10):
             num_packets = random.randrange(2, 100)
             lengths = [random.randrange(10, 10000)
-                       for i in range(num_packets)]
+                       for i in xrange(num_packets)]
             packets = list(map(random_file.read, lengths))
             self.failUnlessEqual(
                 packets, OggPage.to_packets(OggPage.from_packets(packets)))
@@ -295,7 +427,7 @@ class TOggPage(TestCase):
         self.failIfEqual(OggPage(), 12)
 
     def test_find_last(self):
-        pages = [OggPage() for i in range(10)]
+        pages = [OggPage() for i in xrange(10)]
         for i, page in enumerate(pages):
             page.sequence = i
         data = BytesIO(b"".join([page.write() for page in pages]))
@@ -303,7 +435,7 @@ class TOggPage(TestCase):
             OggPage.find_last(data, pages[0].serial), pages[-1])
 
     def test_find_last_really_last(self):
-        pages = [OggPage() for i in range(10)]
+        pages = [OggPage() for i in xrange(10)]
         pages[-1].last = True
         for i, page in enumerate(pages):
             page.sequence = i
@@ -312,7 +444,7 @@ class TOggPage(TestCase):
             OggPage.find_last(data, pages[0].serial), pages[-1])
 
     def test_find_last_muxed(self):
-        pages = [OggPage() for i in range(10)]
+        pages = [OggPage() for i in xrange(10)]
         for i, page in enumerate(pages):
             page.sequence = i
         pages[-2].last = True
@@ -322,7 +454,7 @@ class TOggPage(TestCase):
             OggPage.find_last(data, pages[0].serial), pages[-2])
 
     def test_find_last_no_serial(self):
-        pages = [OggPage() for i in range(10)]
+        pages = [OggPage() for i in xrange(10)]
         for i, page in enumerate(pages):
             page.sequence = i
         data = BytesIO(b"".join([page.write() for page in pages]))
@@ -377,16 +509,15 @@ class TOggPage(TestCase):
 
 class TOggFileTypeMixin(object):
 
+    PADDING_SUPPORT = True
+
     def scan_file(self):
-        fileobj = open(self.filename, "rb")
-        try:
+        with open(self.filename, "rb") as fileobj:
             try:
                 while True:
                     OggPage(fileobj)
             except EOFError:
                 pass
-        finally:
-            fileobj.close()
 
     def test_pprint_empty(self):
         self.audio.pprint()
@@ -446,6 +577,16 @@ class TOggFileTypeMixin(object):
 
         self.scan_file()
 
+    def test_delete_remove_padding(self):
+        if not self.PADDING_SUPPORT:
+            return
+        self.audio.clear()
+        self.audio.save(padding=lambda x: 0)
+        filesize = os.path.getsize(self.audio.filename)
+        self.audio.delete()
+        # deleting shouldn't add padding
+        self.assertTrue(os.path.getsize(self.audio.filename) <= filesize)
+
     def test_really_big(self):
         self.audio["foo"] = "foo" * (2 ** 16)
         self.audio["bar"] = "bar" * (2 ** 16)
@@ -520,6 +661,20 @@ class TOggFileTypeMixin(object):
     def test_mime_secondary(self):
         self.failUnless('application/ogg' in self.audio.mime)
 
+    def test_padding(self):
+        if not self.PADDING_SUPPORT:
+            return
+
+        self.audio.clear()
+        self.audio["foo"] = ["bar"]
+
+        for i in [0, 1, 2, 42, 5000, 4999]:
+            self.audio.save(padding=lambda x: i)
+            new = self.Kind(self.filename)
+            self.assertEqual(new.tags._padding, i)
+            self.assertEqual(new["foo"], ["bar"])
+            self.ogg_reference(self.filename)
+
 
 def call_ogginfo(*args):
     with open(os.devnull, 'wb') as null:
