@@ -175,6 +175,9 @@ class TOggVorbis(TestCase, TOggFileTypeMixin):
     def test_mime(self):
         self.failUnless("audio/vorbis" in self.audio.mime)
 
+    def test_init_padding(self):
+        self.assertEqual(self.audio.tags._padding, 0)
+
 try:
     import ogg.vorbis
 except ImportError:
