@@ -1,25 +1,32 @@
+# -*- coding: utf-8 -*-
+
 import os
 from tempfile import mkstemp
 import shutil
 
-from tests import add
+from mutagen._compat import text_type
+from mutagen._toolsutil import fsnative as fsn
+
 from tests.test_tools import _TTools
+from tests import DATA_DIR
 
 
 class TMOggSPlit(_TTools):
 
-    TOOL_NAME = "moggsplit"
+    TOOL_NAME = u"moggsplit"
 
     def setUp(self):
         super(TMOggSPlit, self).setUp()
-        original = os.path.join('tests', 'data', 'multipagecomment.ogg')
-        fd, self.filename = mkstemp(suffix='.ogg')
+        original = os.path.join(
+            DATA_DIR, fsn(u'multipagecomment.ogg'))
+        fd, self.filename = mkstemp(suffix=fsn(u'.ogg'))
         os.close(fd)
         shutil.copy(original, self.filename)
 
         # append the second file
         first = open(self.filename, "ab")
-        to_append = os.path.join('tests', 'data', 'multipage-setup.ogg')
+        to_append = os.path.join(
+            DATA_DIR, fsn(u'multipage-setup.ogg'))
         second = open(to_append, "rb")
         first.write(second.read())
         second.close()
@@ -31,14 +38,13 @@ class TMOggSPlit(_TTools):
 
     def test_basic(self):
         d = os.path.dirname(self.filename)
-        p = os.path.join(d, "%(stream)d.%(ext)s")
-        res, out = self.call("--pattern", p, self.filename)
+        p = os.path.join(d, fsn(u"%(stream)d.%(ext)s"))
+        res, out = self.call(fsn(u"--pattern"), p, self.filename)
         self.failIf(res)
         self.failIf(out)
 
         for stream in [1002429366, 1806412655]:
-            stream_path = os.path.join(d, str(stream) + ".ogg")
+            stream_path = os.path.join(
+                d, fsn(text_type(stream)) + fsn(u".ogg"))
             self.failUnless(os.path.exists(stream_path))
             os.unlink(stream_path)
-
-add(TMOggSPlit)
