@@ -1,16 +1,19 @@
+# -*- coding: utf-8 -*-
+
 import os
 import glob
 
-from tests import add
 from tests.test_tools import _TTools
 
+from mutagen._toolsutil import fsnative as fsn
+
 
 class TMutagenInspect(_TTools):
 
-    TOOL_NAME = "mutagen-inspect"
+    TOOL_NAME = u"mutagen-inspect"
 
     def test_basic(self):
-        base = os.path.join('tests', 'data')
+        base = os.path.join(fsn(u'tests'), fsn(u'data'))
         self.paths = glob.glob(os.path.join(base, "empty*"))
         self.paths += glob.glob(os.path.join(base, "silence-*"))
 
@@ -20,5 +23,3 @@ class TMutagenInspect(_TTools):
             self.failUnless(out.strip())
             self.failIf("Unknown file type" in out)
             self.failIf("Errno" in out)
-
-add(TMutagenInspect)
