@@ -1,17 +1,18 @@
+# -*- coding: utf-8 -*-
+
 import os
 
-from tests import add
 from tests.test_tools import _TTools
 
+from mutagen._toolsutil import fsnative as fsn
+
 
 class TMutagenPony(_TTools):
 
-    TOOL_NAME = "mutagen-pony"
+    TOOL_NAME = u"mutagen-pony"
 
     def test_basic(self):
-        base = os.path.join('tests', 'data')
+        base = os.path.join(fsn(u'tests'), fsn(u'data'))
         res, out = self.call(base)
         self.failIf(res)
         self.failUnless("Report for %s" % base in out)
-
-add(TMutagenPony)
